"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeQueryAsync = exports.executeBulkQueryAsync = exports.executeToolingQueryAsync = void 0;
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const progressBar_1 = require("../../.../../../ui/progressBar");
const BULK_THRESHOLD = 2000;
class BaseMetadataRetriever {
    constructor(org, tooling = false) {
        this.org = org;
        this.tooling = tooling;
        this.dataLoaded = false;
        this.cacheFileName = "";
        this.objectName = "";
    }
    setQuery(query) {
        this.query = query;
        this.countQuery = this.generateCountQuery();
    }
    generateCountQuery() {
        let queryParts = this.query.toUpperCase().split("FROM");
        let objectParts = queryParts[1].trim().split(" ");
        let objectName = objectParts[0].trim();
        this.objectName = objectName;
        let countQuery = `SELECT COUNT() FROM ${objectName}`;
        return countQuery;
    }
    async getObjects() {
        //let records: T[] = [];
        const conn = this.org.getConnection();
        if (this.tooling) {
            return executeToolingQueryAsync(this.query, conn, this.objectName);
        }
        else {
            let recordsCount = await this.getCount();
            if (recordsCount > BULK_THRESHOLD) {
                return executeBulkQueryAsync(this.query, conn, this.objectName, recordsCount);
            }
            else {
                return executeQueryAsync(this.query, conn, this.objectName);
            }
        }
    }
    async getCount() {
        sfpowerkit_1.SFPowerkit.log(`Count Query: ${this.countQuery}`, core_1.LoggerLevel.TRACE);
        let result = await this.org.getConnection().query(this.countQuery);
        sfpowerkit_1.SFPowerkit.log(`Retrieved count ${result.totalSize}`, core_1.LoggerLevel.TRACE);
        return result.totalSize;
    }
}
exports.default = BaseMetadataRetriever;
async function executeToolingQueryAsync(query, conn, object) {
    let promiseQuery = new Promise((resolve, reject) => {
        let records = [];
        let hasInitProgress = false;
        let progressBar = new progressBar_1.ProgressBar().create(`Querying data from ${object}`, `Records fetched`, core_1.LoggerLevel.DEBUG);
        let queryRun = conn.tooling
            .query(query)
            .on("record", function (record) {
            if (!hasInitProgress) {
                hasInitProgress = true;
                progressBar.start(queryRun.totalSize);
            }
            records.push(record);
            progressBar.increment(1);
        })
            .on("end", function () {
            progressBar.stop();
            resolve(records);
        })
            .on("error", function (error) {
            progressBar.stop();
            reject(error);
        })
            .run({
            autoFetch: true,
            maxFetch: 1000000
        });
    });
    return promiseQuery;
}
exports.executeToolingQueryAsync = executeToolingQueryAsync;
async function executeBulkQueryAsync(query, conn, object, recordCount) {
    let promiseQuery = new Promise((resolve, reject) => {
        let records = [];
        let hasInitProgress = false;
        let progressBar = new progressBar_1.ProgressBar().create(`Querying data from ${object}`, "Records fetched", core_1.LoggerLevel.DEBUG);
        sfpowerkit_1.SFPowerkit.log(`Using Bulk API`, core_1.LoggerLevel.DEBUG);
        conn.bulk
            .query(query)
            .on("record", function (record) {
            if (!hasInitProgress) {
                hasInitProgress = true;
                progressBar.start(recordCount);
            }
            records.push(record);
            progressBar.increment(1);
        })
            .on("end", function () {
            progressBar.stop();
            resolve(records);
        })
            .on("error", function (error) {
            progressBar.stop();
            sfpowerkit_1.SFPowerkit.log(`Error when using bulk api `, core_1.LoggerLevel.ERROR);
            sfpowerkit_1.SFPowerkit.log(error, core_1.LoggerLevel.ERROR);
            reject(error);
        });
    });
    return promiseQuery;
}
exports.executeBulkQueryAsync = executeBulkQueryAsync;
async function executeQueryAsync(query, conn, object) {
    let promiseQuery = new Promise((resolve, reject) => {
        let records = [];
        let hasInitProgress = false;
        let progressBar = new progressBar_1.ProgressBar().create(`Querying data from ${object}`, "Records fetched", core_1.LoggerLevel.DEBUG);
        let queryRun = conn
            .query(query)
            .on("record", function (record) {
            if (!hasInitProgress) {
                hasInitProgress = true;
                progressBar.start(queryRun.totalSize);
            }
            records.push(record);
            progressBar.increment(1);
        })
            .on("end", function () {
            progressBar.stop();
            resolve(records);
        })
            .on("error", function (error) {
            progressBar.stop();
            reject(error);
        })
            .run({
            autoFetch: true,
            maxFetch: 1000000
        });
    });
    return promiseQuery;
}
exports.executeQueryAsync = executeQueryAsync;
//# sourceMappingURL=baseMetadataRetriever.js.map